@@ -13,17 +13,17 @@ log = logging.getLogger(__name__)
 ###################################################
 # public
 def compare_metadata(context, metadata):
-    """Compare a database schema to that given in a :class:`~sqlalchemy.schema.MetaData`
-    instance.
-    
+    """Compare a database schema to that given in a
+    :class:`~sqlalchemy.schema.MetaData` instance.
+
     The database connection is presented in the context
-    of a :class:`.MigrationContext` object, which 
+    of a :class:`.MigrationContext` object, which
     provides database connectivity as well as optional
     comparison functions to use for datatypes and
     server defaults - see the "autogenerate" arguments
     at :meth:`.EnvironmentContext.configure`
     for details on these.
-    
+
     The return format is a list of "diff" directives,
     each representing individual differences::
 
@@ -31,7 +31,7 @@ def compare_metadata(context, metadata):
         from alembic.autogenerate import compare_metadata
         from sqlalchemy.schema import SchemaItem
         from sqlalchemy.types import TypeEngine
-        from sqlalchemy import (create_engine, MetaData, Column, 
+        from sqlalchemy import (create_engine, MetaData, Column,
                 Integer, String, Table)
         import pprint
 
@@ -50,12 +50,12 @@ def compare_metadata(context, metadata):
             )''')
 
         metadata = MetaData()
-        Table('foo', metadata, 
+        Table('foo', metadata,
             Column('id', Integer, primary_key=True),
             Column('data', Integer),
             Column('x', Integer, nullable=False)
         )
-        Table('bat', metadata, 
+        Table('bat', metadata,
             Column('info', String)
         )
 
@@ -63,13 +63,15 @@ def compare_metadata(context, metadata):
 
         diff = compare_metadata(mc, metadata)
         pprint.pprint(diff, indent=2, width=20)
-    
+
     Output::
-    
+
         [ ( 'add_table',
-            Table('bat', MetaData(bind=None), Column('info', String(), table=<bat>), schema=None)),
+            Table('bat', MetaData(bind=None),
+                Column('info', String(), table=<bat>), schema=None)),
           ( 'remove_table',
-            Table(u'bar', MetaData(bind=None), Column(u'data', VARCHAR(), table=<bar>), schema=None)),
+            Table(u'bar', MetaData(bind=None),
+                Column(u'data', VARCHAR(), table=<bar>), schema=None)),
           ( 'add_column',
             'foo',
             Column('data', Integer(), table=<foo>)),
@@ -83,13 +85,13 @@ def compare_metadata(context, metadata):
                 'existing_type': INTEGER()},
               True,
               False)]]
-    
-    
+
+
     :param context: a :class:`.MigrationContext`
      instance.
-    :param metadata: a :class:`~sqlalchemy.schema.MetaData` 
+    :param metadata: a :class:`~sqlalchemy.schema.MetaData`
      instance.
-    
+
     """
     autogen_context, connection = _autogen_context(context, None)
     diffs = []
@@ -99,7 +101,8 @@ def compare_metadata(context, metadata):
 ###################################################
 # top level
 
-def _produce_migration_diffs(context, template_args, imports):
+def _produce_migration_diffs(context, template_args,
+                                imports, _include_only=()):
     opts = context.opts
     metadata = opts['target_metadata']
     if metadata is None:
@@ -112,7 +115,8 @@ def _produce_migration_diffs(context, template_args, imports):
     autogen_context, connection = _autogen_context(context, imports)
 
     diffs = []
-    _produce_net_changes(connection, metadata, diffs, autogen_context)
+    _produce_net_changes(connection, metadata, diffs,
+                                autogen_context, _include_only)
     template_args[opts['upgrade_token']] = \
             _indent(_produce_upgrade_commands(diffs, autogen_context))
     template_args[opts['downgrade_token']] = \
@@ -123,15 +127,16 @@ def _autogen_context(context, imports):
     opts = context.opts
     connection = context.bind
     return {
-        'imports':imports,
-        'connection':connection,
-        'dialect':connection.dialect,
-        'context':context,
-        'opts':opts
+        'imports': imports,
+        'connection': connection,
+        'dialect': connection.dialect,
+        'context': context,
+        'opts': opts
     }, connection
 
 def _indent(text):
-    text = "### commands auto generated by Alembic - please adjust! ###\n" + text
+    text = "### commands auto generated by Alembic - "\
+                    "please adjust! ###\n" + text
     text += "\n### end Alembic commands ###"
     text = re.compile(r'^', re.M).sub("    ", text).strip()
     return text
@@ -139,17 +144,22 @@ def _indent(text):
 ###################################################
 # walk structures
 
-def _produce_net_changes(connection, metadata, diffs, autogen_context):
+def _produce_net_changes(connection, metadata, diffs, autogen_context,
+                            include_only=None):
     inspector = Inspector.from_engine(connection)
     # TODO: not hardcode alembic_version here ?
     conn_table_names = set(inspector.get_table_names()).\
                             difference(['alembic_version'])
-    metadata_table_names = OrderedSet([table.name for table in metadata.sorted_tables])
+    if include_only:
+        conn_table_names = conn_table_names.intersection(include_only)
+
+    metadata_table_names = OrderedSet([table.name
+                                for table in metadata.sorted_tables])
 
     _compare_tables(conn_table_names, metadata_table_names,
                     inspector, metadata, diffs, autogen_context)
 
-def _compare_tables(conn_table_names, metadata_table_names, 
+def _compare_tables(conn_table_names, metadata_table_names,
                     inspector, metadata, diffs, autogen_context):
     for tname in metadata_table_names.difference(conn_table_names):
         diffs.append(("add_table", metadata.tables[tname]))
@@ -167,7 +177,7 @@ def _compare_tables(conn_table_names, metadata_table_names,
     existing_tables = conn_table_names.intersection(metadata_table_names)
 
     conn_column_info = dict(
-        (tname, 
+        (tname,
             dict(
                 (rec["name"], rec)
                 for rec in inspector.get_columns(tname)
@@ -177,12 +187,12 @@ def _compare_tables(conn_table_names, metadata_table_names,
     )
 
     for tname in sorted(existing_tables):
-        _compare_columns(tname, 
-                conn_column_info[tname], 
+        _compare_columns(tname,
+                conn_column_info[tname],
                 metadata.tables[tname],
                 diffs, autogen_context)
 
-    # TODO: 
+    # TODO:
     # index add/drop
     # table constraints
     # sequences
@@ -190,7 +200,8 @@ def _compare_tables(conn_table_names, metadata_table_names,
 ###################################################
 # element comparison
 
-def _compare_columns(tname, conn_table, metadata_table, diffs, autogen_context):
+def _compare_columns(tname, conn_table, metadata_table,
+                                diffs, autogen_context):
     metadata_cols_by_name = dict((c.name, c) for c in metadata_table.c)
     conn_col_names = set(conn_table)
     metadata_col_names = set(metadata_cols_by_name)
@@ -234,37 +245,39 @@ def _compare_columns(tname, conn_table, metadata_table, diffs, autogen_context):
         if col_diff:
             diffs.append(col_diff)
 
-def _compare_nullable(tname, cname, conn_col, 
-                            metadata_col_nullable, diffs, 
+def _compare_nullable(tname, cname, conn_col,
+                            metadata_col_nullable, diffs,
                             autogen_context):
     conn_col_nullable = conn_col['nullable']
     if conn_col_nullable is not metadata_col_nullable:
         diffs.append(
-            ("modify_nullable", tname, cname, 
+            ("modify_nullable", tname, cname,
                 {
-                    "existing_type":conn_col['type'],
-                    "existing_server_default":conn_col['default'],
+                    "existing_type": conn_col['type'],
+                    "existing_server_default": conn_col['default'],
                 },
-                conn_col_nullable, 
+                conn_col_nullable,
                 metadata_col_nullable),
         )
-        log.info("Detected %s on column '%s.%s'", 
+        log.info("Detected %s on column '%s.%s'",
             "NULL" if metadata_col_nullable else "NOT NULL",
             tname,
             cname
         )
 
-def _compare_type(tname, cname, conn_col, 
-                            metadata_col, diffs, 
+def _compare_type(tname, cname, conn_col,
+                            metadata_col, diffs,
                             autogen_context):
 
     conn_type = conn_col['type']
     metadata_type = metadata_col.type
     if conn_type._type_affinity is sqltypes.NullType:
-        log.info("Couldn't determine database type for column '%s.%s'" % (tname, cname))
+        log.info("Couldn't determine database type "
+                    "for column '%s.%s'" % (tname, cname))
         return
     if metadata_type._type_affinity is sqltypes.NullType:
-        log.info("Column '%s.%s' has no type within the model; can't compare" % (tname, cname))
+        log.info("Column '%s.%s' has no type within "
+                        "the model; can't compare" % (tname, cname))
         return
 
     isdiff = autogen_context['context']._compare_type(conn_col, metadata_col)
@@ -272,26 +285,27 @@ def _compare_type(tname, cname, conn_col,
     if isdiff:
 
         diffs.append(
-            ("modify_type", tname, cname, 
+            ("modify_type", tname, cname,
                     {
-                        "existing_nullable":conn_col['nullable'],
-                        "existing_server_default":conn_col['default'],
+                        "existing_nullable": conn_col['nullable'],
+                        "existing_server_default": conn_col['default'],
                     },
-                    conn_type, 
+                    conn_type,
                     metadata_type),
         )
-        log.info("Detected type change from %r to %r on '%s.%s'", 
+        log.info("Detected type change from %r to %r on '%s.%s'",
             conn_type, metadata_type, tname, cname
         )
 
-def _compare_server_default(tname, cname, conn_col, metadata_col, 
+def _compare_server_default(tname, cname, conn_col, metadata_col,
                                 diffs, autogen_context):
 
     metadata_default = metadata_col.server_default
     conn_col_default = conn_col['default']
     if conn_col_default is None and metadata_default is None:
         return False
-    rendered_metadata_default = _render_server_default(metadata_default, autogen_context)
+    rendered_metadata_default = _render_server_default(
+                            metadata_default, autogen_context)
     isdiff = autogen_context['context']._compare_server_default(
                         conn_col, metadata_col,
                         rendered_metadata_default
@@ -299,15 +313,15 @@ def _compare_server_default(tname, cname, conn_col, metadata_col,
     if isdiff:
         conn_col_default = conn_col['default']
         diffs.append(
-            ("modify_default", tname, cname, 
+            ("modify_default", tname, cname,
                 {
-                    "existing_nullable":conn_col['nullable'],
-                    "existing_type":conn_col['type'],
+                    "existing_nullable": conn_col['nullable'],
+                    "existing_type": conn_col['type'],
                 },
                 conn_col_default,
                 metadata_default),
         )
-        log.info("Detected server default on column '%s.%s'", 
+        log.info("Detected server default on column '%s.%s'",
             tname,
             cname
         )
@@ -345,8 +359,8 @@ def _invoke_adddrop_command(updown, args, autogen_context):
     cmd_args = args[1:] + (autogen_context,)
 
     _commands = {
-        "table":(_drop_table, _add_table),
-        "column":(_drop_column, _add_column),
+        "table": (_drop_table, _add_table),
+        "column": (_drop_column, _add_column),
     }
 
     cmd_callables = _commands[cmd_type]
@@ -365,9 +379,9 @@ def _invoke_modify_command(updown, args, autogen_context):
     kw = {}
 
     _arg_struct = {
-        "modify_type":("existing_type", "type_"),
-        "modify_nullable":("existing_nullable", "nullable"),
-        "modify_default":("existing_server_default", "server_default"),
+        "modify_type": ("existing_type", "type_"),
+        "modify_nullable": ("existing_nullable", "nullable"),
+        "modify_default": ("existing_server_default", "server_default"),
     }
     for diff in args:
         diff_kw = diff[3]
@@ -395,12 +409,12 @@ def _invoke_modify_command(updown, args, autogen_context):
 
 def _add_table(table, autogen_context):
     return "%(prefix)screate_table(%(tablename)r,\n%(args)s\n)" % {
-        'tablename':table.name,
-        'prefix':_alembic_autogenerate_prefix(autogen_context),
-        'args':',\n'.join(
+        'tablename': table.name,
+        'prefix': _alembic_autogenerate_prefix(autogen_context),
+        'args': ',\n'.join(
             [_render_column(col, autogen_context) for col in table.c] +
-            sorted([rcons for rcons in 
-                [_render_constraint(cons, autogen_context) for cons in 
+            sorted([rcons for rcons in
+                [_render_constraint(cons, autogen_context) for cons in
                     table.constraints]
                 if rcons is not None
             ])
@@ -409,25 +423,25 @@ def _add_table(table, autogen_context):
 
 def _drop_table(table, autogen_context):
     return "%(prefix)sdrop_table(%(tname)r)" % {
-            "prefix":_alembic_autogenerate_prefix(autogen_context),
-            "tname":table.name
+            "prefix": _alembic_autogenerate_prefix(autogen_context),
+            "tname": table.name
         }
 
 def _add_column(tname, column, autogen_context):
     return "%(prefix)sadd_column(%(tname)r, %(column)s)" % {
-            "prefix":_alembic_autogenerate_prefix(autogen_context),
-            "tname":tname,
-            "column":_render_column(column, autogen_context)
+            "prefix": _alembic_autogenerate_prefix(autogen_context),
+            "tname": tname,
+            "column": _render_column(column, autogen_context)
             }
 
 def _drop_column(tname, column, autogen_context):
     return "%(prefix)sdrop_column(%(tname)r, %(cname)r)" % {
-            "prefix":_alembic_autogenerate_prefix(autogen_context),
-            "tname":tname,
-            "cname":column.name
+            "prefix": _alembic_autogenerate_prefix(autogen_context),
+            "tname": tname,
+            "cname": column.name
             }
 
-def _modify_col(tname, cname, 
+def _modify_col(tname, cname,
                 autogen_context,
                 server_default=False,
                 type_=None,
@@ -438,27 +452,30 @@ def _modify_col(tname, cname,
     sqla_prefix = _sqlalchemy_autogenerate_prefix(autogen_context)
     indent = " " * 11
     text = "%(prefix)salter_column(%(tname)r, %(cname)r" % {
-                            'prefix':_alembic_autogenerate_prefix(autogen_context), 
-                            'tname':tname, 
-                            'cname':cname}
-    text += ", \n%sexisting_type=%s" % (indent, 
+                            'prefix': _alembic_autogenerate_prefix(
+                                                autogen_context),
+                            'tname': tname,
+                            'cname': cname}
+    text += ",\n%sexisting_type=%s" % (indent,
                     _repr_type(sqla_prefix, existing_type, autogen_context))
     if server_default is not False:
-        text += ", \n%sserver_default=%s" % (indent, 
-                        _render_server_default(server_default, autogen_context),)
+        text += ",\n%sserver_default=%s" % (indent,
+                        _render_server_default(
+                                server_default, autogen_context),)
     if type_ is not None:
-        text += ", \n%stype_=%s" % (indent, _repr_type(sqla_prefix, type_, autogen_context))
+        text += ",\n%stype_=%s" % (indent,
+                        _repr_type(sqla_prefix, type_, autogen_context))
     if nullable is not None:
-        text += ", \n%snullable=%r" % (
+        text += ",\n%snullable=%r" % (
                         indent, nullable,)
     if existing_nullable is not None:
-        text += ", \n%sexisting_nullable=%r" % (
+        text += ",\n%sexisting_nullable=%r" % (
                         indent, existing_nullable)
     if existing_server_default:
-        text += ", \n%sexisting_server_default=%s" % (
-                        indent, 
+        text += ",\n%sexisting_server_default=%s" % (
+                        indent,
                         _render_server_default(
-                            existing_server_default, 
+                            existing_server_default,
                             autogen_context),
                     )
     text += ")"
@@ -473,17 +490,20 @@ def _alembic_autogenerate_prefix(autogen_context):
 def _render_column(column, autogen_context):
     opts = []
     if column.server_default:
-        opts.append(("server_default", 
-                    _render_server_default(column.server_default, autogen_context)))
+        opts.append(("server_default",
+                    _render_server_default(
+                            column.server_default, autogen_context
+                    )))
     if column.nullable is not None:
         opts.append(("nullable", column.nullable))
 
     # TODO: for non-ascii colname, assign a "key"
     return "%(prefix)sColumn(%(name)r, %(type)s, %(kw)s)" % {
-        'prefix':_sqlalchemy_autogenerate_prefix(autogen_context),
-        'name':column.name,
-        'type':_repr_type(_sqlalchemy_autogenerate_prefix(autogen_context), column.type, autogen_context),
-        'kw':", ".join(["%s=%s" % (kwname, val) for kwname, val in opts])
+        'prefix': _sqlalchemy_autogenerate_prefix(autogen_context),
+        'name': column.name,
+        'type': _repr_type(_sqlalchemy_autogenerate_prefix(autogen_context),
+                                column.type, autogen_context),
+        'kw': ", ".join(["%s=%s" % (kwname, val) for kwname, val in opts])
     }
 
 def _render_server_default(default, autogen_context):
@@ -491,13 +511,14 @@ def _render_server_default(default, autogen_context):
         if isinstance(default.arg, basestring):
             default = default.arg
         else:
-            default = str(default.arg.compile(dialect=autogen_context['dialect']))
+            default = str(default.arg.compile(
+                            dialect=autogen_context['dialect']))
     if isinstance(default, basestring):
-        # TODO: this is just a hack to get 
+        # TODO: this is just a hack to get
         # tests to pass until we figure out
         # WTF sqlite is doing
         default = re.sub(r"^'|'$", "", default)
-        return "'%s'" % default
+        return repr(default)
     else:
         return None
 
@@ -524,8 +545,8 @@ def _render_primary_key(constraint, autogen_context):
     if constraint.name:
         opts.append(("name", repr(constraint.name)))
     return "%(prefix)sPrimaryKeyConstraint(%(args)s)" % {
-        "prefix":_sqlalchemy_autogenerate_prefix(autogen_context),
-        "args":", ".join(
+        "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
+        "args": ", ".join(
             [repr(c.key) for c in constraint.columns] +
             ["%s=%s" % (kwname, val) for kwname, val in opts]
         ),
@@ -536,25 +557,52 @@ def _render_foreign_key(constraint, autogen_context):
     if constraint.name:
         opts.append(("name", repr(constraint.name)))
     # TODO: deferrable, initially, etc.
-    return "%(prefix)sForeignKeyConstraint([%(cols)s], [%(refcols)s], %(args)s)" % {
-        "prefix":_sqlalchemy_autogenerate_prefix(autogen_context),
-        "cols":", ".join("'%s'" % f.parent.key for f in constraint.elements),
-        "refcols":", ".join(repr(f._get_colspec()) for f in constraint.elements),
-        "args":", ".join(
+    return "%(prefix)sForeignKeyConstraint([%(cols)s], "\
+            "[%(refcols)s], %(args)s)" % {
+        "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
+        "cols": ", ".join("'%s'" % f.parent.key for f in constraint.elements),
+        "refcols": ", ".join(repr(f._get_colspec())
+                            for f in constraint.elements),
+        "args": ", ".join(
             ["%s=%s" % (kwname, val) for kwname, val in opts]
         ),
     }
 
 def _render_check_constraint(constraint, autogen_context):
+    # detect the constraint being part of
+    # a parent type which is probably in the Table already.
+    # ideally SQLAlchemy would give us more of a first class
+    # way to detect this.
+    if constraint._create_rule and \
+        hasattr(constraint._create_rule, 'target') and \
+        isinstance(constraint._create_rule.target,
+                sqltypes.TypeEngine):
+        return None
     opts = []
     if constraint.name:
         opts.append(("name", repr(constraint.name)))
-    return "%(prefix)sCheckConstraint('TODO')" % {
-            "prefix":_sqlalchemy_autogenerate_prefix(autogen_context)
+    return "%(prefix)sCheckConstraint(%(sqltext)r)" % {
+            "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
+            "sqltext": str(
+                constraint.sqltext.compile(
+                    dialect=autogen_context['dialect']
+                )
+            )
         }
 
+def _render_unique_constraint(constraint, autogen_context):
+    opts = []
+    if constraint.name:
+        opts.append(("name", "'%s'" % constraint.name))
+    return "%(prefix)sUniqueConstraint(%(cols)s%(opts)s)" % {
+        'opts': ", " + (", ".join("%s=%s" % (k, v)
+                            for k, v in opts)) if opts else "",
+        'cols': ",".join(["'%s'" % c.name for c in constraint.columns]),
+        "prefix": _sqlalchemy_autogenerate_prefix(autogen_context)
+        }
 _constraint_renderers = {
-    schema.PrimaryKeyConstraint:_render_primary_key,
-    schema.ForeignKeyConstraint:_render_foreign_key,
-    schema.CheckConstraint:_render_check_constraint
+    schema.PrimaryKeyConstraint: _render_primary_key,
+    schema.ForeignKeyConstraint: _render_foreign_key,
+    schema.UniqueConstraint: _render_unique_constraint,
+    schema.CheckConstraint: _render_check_constraint
 }
