@@ -74,7 +74,7 @@ class Operations(object):
         t = schema.Table(source, schema.MetaData(), 
                     *[schema.Column(n, NULLTYPE) for n in local_cols])
         kw['name'] = name
-        uq = schema.UniqueConstraint(*t.c, **kw)
+        uq = schema.UniqueConstraint(*[t.c[n] for n in local_cols], **kw)
         # TODO: need event tests to ensure the event
         # is fired off here
         t.append_constraint(uq)
@@ -109,9 +109,9 @@ class Operations(object):
 
         """
         if isinstance(fk._colspec, basestring):
-            table_key, cname = fk._colspec.split('.')
+            table_key, cname = fk._colspec.rsplit('.', 1)
             if '.' in table_key:
-                tokens = tname.split('.')
+                tokens = table_key.split('.')
                 sname = ".".join(tokens[0:-1])
                 tname = tokens[-1]
             else:
@@ -270,6 +270,20 @@ class Operations(object):
                 Column('account_id', INTEGER, ForeignKey('accounts.id'))
             )
 
+        Note that this statement uses the :class:`~sqlalchemy.schema.Column`
+        construct as is from the SQLAlchemy library.  In particular,
+        default values to be created on the database side are
+        specified using the ``server_default`` parameter, and not
+        ``default`` which only specifies Python-side defaults::
+        
+            from alembic import op
+            from sqlalchemy import Column, TIMESTAMP, func
+
+            # specify "DEFAULT NOW" along with the column add
+            op.add_column('account',
+                Column('timestamp', TIMESTAMP, server_default=func.now())
+            )
+
         :param table_name: String name of the parent table.
         :param column: a :class:`sqlalchemy.schema.Column` object
          representing the new column.
@@ -459,10 +473,26 @@ class Operations(object):
             from alembic import op
 
             op.create_table(
-                'accounts',
+                'account',
                 Column('id', INTEGER, primary_key=True),
                 Column('name', VARCHAR(50), nullable=False),
                 Column('description', NVARCHAR(200))
+                Column('timestamp', TIMESTAMP, server_default=func.now())
+            )
+
+        Note that :meth:`.create_table` accepts :class:`~sqlalchemy.schema.Column`
+        constructs directly from the SQLAlchemy library.  In particular,
+        default values to be created on the database side are
+        specified using the ``server_default`` parameter, and not
+        ``default`` which only specifies Python-side defaults::
+        
+            from alembic import op
+            from sqlalchemy import Column, TIMESTAMP, func
+
+            # specify "DEFAULT NOW" along with the "timestamp" column
+            op.create_table('account',
+                Column('id', INTEGER, primary_key=True),
+                Column('timestamp', TIMESTAMP, server_default=func.now())
             )
 
         :param name: Name of the table
@@ -529,10 +559,30 @@ class Operations(object):
         # 0.7.6 and further raises on Index with no columns
         self.impl.drop_index(self._index(name, tablename, ['x']))
 
-    def drop_constraint(self, name, tablename):
-        """Drop a constraint of the given name"""
+    def drop_constraint(self, name, tablename, type=None):
+        """Drop a constraint of the given name, typically via DROP CONSTRAINT.
+        
+        :param name: name of the constraint.
+        :param tablename: tablename.
+        :param type: optional, required on MySQL.  can be 
+        'foreignkey', 'unique', or 'check'
+
+        """
         t = self._table(tablename)
-        const = schema.Constraint(name=name)
+        types = {
+            'foreignkey':lambda name:schema.ForeignKeyConstraint(
+                                [], [], name=name),
+            'unique':schema.UniqueConstraint,
+            'check':lambda name:schema.CheckConstraint("", name=name),
+            None:schema.Constraint
+        }
+        try:
+            const = types[type]
+        except KeyError:
+            raise TypeError("'type' can be one of %s" % 
+                        ", ".join(sorted(repr(x) for x in types))) 
+
+        const = const(name=name)
         t.append_constraint(const)
         self.impl.drop_constraint(const)
 
